#!/usr/bin/bash

unset -v _appdir
unset -v _appbackupdir

source ${_cfgdir}/sigpdv.conf

: ${_appuid:=777}
: ${_appuser:=sigpdv}
: ${_appgroup:=sigpdv}
: ${_localusers:=${localusers}}
: ${_defpass:=${defpass}}

sigpdv.backup() {
    function _search() {
	local _defpath=/usr/local/sigpdv
	local _path __path

        declare -a _SEARCH_PATH=(
            '/usr/local/sigpdv'
            '/usr/share/sigpdv'
            '/opt/sig/sigpdv'
            '/opt/sigpdv'
            '/sigpdv'
            '/app'
        )

        printf \
            $"%s [INFO] Searching for installed version: " \
	    ${product,,}
	for __path in ${_SEARCH_PATH[@]}; do
            [[ -d ${__path} ]] && \
		_path=${__path} && \
   		break
        done
        _appdir=${_path:-${_defpath}}
        _appbackupdir=${_path}.bak

        [[ ! -z "${_path}" ]] \
            && printf $"%s [%s]\n" ${_path} $"found" \
            || printf "[%s]\n" $"not found"
       
        return 0
    }

    function _create_backup() {
        if [[ -r ${_appdir}/pdvconfig.cfg ]]; then
            rm -rf ${_appbackupdir} 2>/dev/null
            mkdir -p /backup/

            printf \
                $"%s [INFO] Creating backup of config files: " \
                ${product,,}
            _date=$(date +%Y%m%d%H%M%S) && \
            _file=${product,,}-cfg-${_date}.tar.gz && \
            cd ${_appdir} && \
            find . \
                -regextype egrep \
                -regex '.*\.(ini|cfg|conf)' \
                | xargs tar -czf \
                    /backup/${_file} \
                    2>/dev/null \
                && printf "%s [%s]\n" /backup/${_file} $"success" \
                || printf "[%s]\n" "failure"; \
                cd ${OLDPWD}

            printf \
                $"%s [INFO] Creating a full backup of %s: " \
                ${product,,} \
                ${product}
            cp -a ${_appdir} ${_appbackupdir} \
                && printf "%s [%s]\n" ${_appbackupdir} $"success" \
                || printf "[%s]\n" $"failure"
        fi
    }

    _search
    _create_backup
}

sigpdv.install() {
    function _install_deps() {
	local _suite=$(lsb_release -cs)
        printf \
            $"%s [INFO] Installing dependencies for %s...\n\n" \
            ${product,,} ${product^^}

	echo "deb http://apt.postgresql.org/pub/repos/apt ${_suite}-pgdg main" \
	       > /etc/apt/sources.list.d/pgdg.list \
	    && wget -qcO - https://www.postgresql.org/media/keys/ACCC4CF8.asc \
	       | apt-key add - > /dev/null 2>&1 && \
        echo "deb https://download.sigsolucoes.net.br/apt ${_suite} main" \
               > /etc/apt/sources.list.d/sig.list \
            && wget -qc http://download.sigsolucoes.net.br/apt/sig-keyring-2016-archive.gpg \
               -O /etc/apt/trusted.gpg.d/sig-keyring-2016-archive.gpg \
            && apt-get -qq update \
            && DEBIAN_FRONTEND=noninteractive apt-get -yqq \
                install --no-install-suggests --no-install-recommends \
                    lib32stdc++6 \
                    libclisitef32 \
                    libclisitef64 \
                    libconvecf \
                    libdarumaframework \
                    libemv \
                    libemv64 \
                    liblebin \
                    libleituramfdbin \
                    libseppemv \
                    libseppemv64 \
                    libswmfd \
                    postgresql-13 \
                    pyside-tools \
                    python-beautifulsoup \
                    python-crypto \
                    python-dateutil \
                    python-elixir=0.7.1-2 \
                    python-geraldo \
                    python-kiwi \
                    python-libxml2 \
                    python-lxml \
                    python-m2crypto \
                    python-mysqldb \
                    python-openssl \
                    python-ordereddict \
                    python-pip \
                    python-psycopg2 \
                    python-pyside \
                    python-pyxmlsec \
                    python-qt4 \
                    python-reportlab \
                    python-requests \
                    python-serial=2.6-nmu1build1 \
                    python-setuptools \
                    python-sig-runtime \
                    python-sqlalchemy-ext=0.8.4-nmu1build1 \
                    python-sqlalchemy=0.8.4-nmu1build1 \
                    python-tz \
                    stoqdrivers \
            && DEBIAN_FRONTEND=noninteractive apt-get -yqq \
                purge \
                    sigpdv-nfce-libs \
            && DEBIAN_FRONTEND=noninteractive apt-mark -qqq \
                hold \
                    python-elixir=0.7.1-2 \
                    python-serial=2.6-nmu1build1 \
                    python-sqlalchemy-ext=0.8.4-nmu1build1 \
                    python-sqlalchemy=0.8.4-nmu1build1 \
            && pip install \
                    segno \
            && pip install \
		--no-deps \
		--no-binary :all: \
                    https://download.sigsolucoes.net.br/produtos/releases/sigutils/2.4/sigutils-${sigutils}.tar.gz \
                    https://download.sigsolucoes.net.br/produtos/releases/signfe/4.51/nfe-${signfe}.tar.gz \
            && _prefix=$(dirname ${_appdir}) \
            && echo "import sys; sys.path.append('${_prefix}');" \
                > /usr/local/lib/python2.7/dist-packages/sigpdv.pth
            echo
    }

    function _download() {
        local _url=$(dirname ${source})
        local _source=$(basename ${1:-${source}})
        local _status

        rm -rf /tmp/${_source}

        printf \
            $"%s [INFO] Downloading source into: %s " \
            ${product,,} /tmp/${_source}

        _status=$(
            wget \
                -qcO /tmp/${_source} \
                ${_url}/${_source} \
                echo ${?}
        )
        
        [[ ${_status} -eq 0 ]] \
            && printf "[%s]\n" $"success" \
            || printf "[%s]\n" $"failure"
        
        return ${_status}
    }

    function _verify_downloads() {
        local _source=$(basename ${1:-${source}})
        local _status

        printf \
            $"%s [INFO] Verifying downloaded sources: " \
            ${product,,}

        _status=$(
            cd /tmp
            sha256sum --quiet \
                -c ${_source}.checksum; \
            echo ${?}
            cd ${OLDPWD}
        )

        [[ ${_status} -eq 0 ]] \
            && printf "[%s]\n" $"success" \
            || printf "[%s]\n" $"failure"
        
        return ${_status}
    }

    function _install() {
        local _source=$(basename ${1:-${source}})
        local _lnkdir=/usr/local/sigpdv
        local _status

        printf \
            $"%s [INFO] Unpacking new version into: %s: " \
            ${product,,} ${_appdir}

        _status=$(
            if [[ ! "${_appdir}" == "${_lnkdir}" ]]; then
                rm -rf ${_lnkdir} && \
                ln -snf \
                    ${_appdir} \
                    ${_lnkdir}
            fi
            rm -rf ${_appdir} && \
            mkdir -p ${_appdir} && \
            tar -C ${_appdir} \
                --transform="s#${product,,}-${serie}.${version}#.#g" \
                -xzf /tmp/${_source} && \
	        cp -f ${_appdir}/pdvconfig.cfg.dist \
	    	${_appdir}/pdvconfig.cfg; \
            echo ${?}
        )

        [[ ${_status} -eq 0 ]] \
            && printf "[%s]\n" $"success" \
            || printf "[%s]\n" $"failure"
        
        return ${_status}
    }

    function _restore_config() {
        local _file=$(ls -Art /backup/${product,,}-cfg* | tail -n 1 2>/dev/null)
        local _status

        if [[ -f "${_file}" ]]; then
            printf \
                $"%s [INFO] Restoring latest configuration backup: " \
                ${product,,}
            _status=$(
                tar -C ${_appdir} \
                    -xzf ${_file}; \
                echo ${?}
            )
            [[ ${_status} -eq 0 ]] \
                && printf "%s [%s]\n" ${_file} $"success" \
                || printf "%s [%s]\n" ${_file} $"failure"
        fi
        return ${_status}
    }

    function _createusers_and_groups() {

        (
            printf \
                $"%s [INFO] Setting up system group '%s': " \
                ${product,,} ${_appgroup}

            if [[ 0$(id -g ${_appgroup} 2>/dev/null) -eq 0 ]]; then
            	groupadd \
                    -g ${_appuid} -r \
                    ${_appgroup} \
                    > /dev/null \
	            	2>&1 \
                && printf "[%s]\n" $"success" \
                || printf "[%s]\n" $"failed"
            else
                printf "[%s]\n" $"skipped"
            fi
        ) && \
        (        
            printf \
                $"%s [INFO] Setting up system user '%s': " \
                ${product,,} ${_appuser}
            
            if [[ 0$(id -u ${_appuser} 2>/dev/null) -eq 0 ]]; then
                useradd \
                    -r -s /sbin/nologin \
                    -M -d ${_appdir} \
                    -u ${_appuid} -g ${_appuid} \
                    ${_appuser} \
	    	        > /dev/null \
		            2>&1 \
                && printf "[%s]\n" $"success" \
                || printf "[%s]\n" $"failed"
            else
                printf "[%s]\n" $"skipped"
            fi
        )

        printf $"%s [INFO] Creating local users and groups: \n" \
            ${product,,}
        for _user in ${_localusers}; do
            if [[ 0$(id -u ${_user} 2>/dev/null) -eq 0 ]]; then
                useradd \
                    -s /bin/bash \
                    -m -d /home/${_user} \
                    -G ${_appgroup} \
                    ${_user} \
                    > /dev/null \
                    2>&1 \
                && echo ${_user}:${_defpass} | \
                    chpasswd \
                    ${_user} \
                && printf "* User %s [%s]\n" ${_user} $"crated" \
                || printf "* User %s [%s]\n" ${_user} $"failed"
            else
                printf "* User %s [%s]\n" ${_user} $"skipped"
            fi
        done
        return 0
    }

    function _fixperms() {
        local _status

        printf \
            $"%s [INFO] Adjusting owerships and permissions: " \
            ${product,,}

        _status=$(
            chown -R ${_appuser}:${_appgroup} ${_appdir} && \
            find ${_appdir} \
                -type d \
                -exec chmod 0770 {} \; && \
            find ${_appdir} \
                -type f \
                -exec chmod 0660 {} \; && \
            find ${_appdir} \
                -type f \
                -regextype egrep \
                -regex '.*\.(py[c]*|so)' \
                -exec chmod 0770 {} \;
                echo ${?}
        )

        [[ ${_status} -eq 0 ]] \
            && printf "[%s]\n" $"success" \
            || printf "[%s]\n" $"failure"
        return ${_status}
    }

    function _cleanup() {
        local _file=$(ls -Art /backup/${product,,}-cfg* | tail -n 1)

        find /backup/ -type f -not -name $(basename ${_file}) -delete
        rm -rf \
            ${_appbackupdir} \
            ${_appdir}/MANIFEST.in \
            ${_appdir}/PKG-INFO \
            ${_appdir}/debian \
            ${_appdir}/setup.cfg \
            ${_appdir}/setup.py \
            ${_appdir}/sigpdv.egg-info
    }

    _download ${source}.checksum
    _download ${source}
    _verify_downloads
    _install_deps
    _install
    _restore_config
    _createusers_and_groups
    _fixperms
    _cleanup
}

# vim: ts=4:sw=4:sts=4:et
